@@ -12,7 +12,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tkWindow.c,v 1.56.2.2 2004/02/16 00:42:34 wolfsuit Exp $
+ * RCS: @(#) $Id: tkWindow.c,v 1.56.2.5 2005/01/07 11:16:29 dkf Exp $
  */
 
 #include "tkPort.h"
@@ -353,14 +353,14 @@ CreateTopLevelWindow(interp, parent, name, screenName, flags)
 	 */
     
 	Tk_CreatePhotoImageFormat(&tkImgFmtGIF);
-	Tk_CreateOldPhotoImageFormat(&tkImgFmtPPM);
+	Tk_CreatePhotoImageFormat(&tkImgFmtPPM);
 
 	/*
 	 * Create exit handler to delete all windows when the application
 	 * exits.
 	 */
 
-	Tcl_CreateExitHandler(DeleteWindowsExitProc, (ClientData) NULL);
+	TkCreateExitHandler(DeleteWindowsExitProc, (ClientData) tsdPtr);
     }
 
     if ((parent != NULL) && (screenName != NULL) && (screenName[0] == '\0')) {
@@ -485,6 +485,7 @@ GetScreen(interp, screenName, screenPtr)
 	     */
 	    dispPtr = TkpOpenDisplay(screenName);
 	    if (dispPtr == NULL) {
+		Tcl_ResetResult(interp);
 		Tcl_AppendResult(interp, "couldn't connect to display \"",
 			screenName, "\"", (char *) NULL);
 		return (TkDisplay *) NULL;
@@ -2705,12 +2706,11 @@ Tk_GetNumMainWindows()
 
 static void
 DeleteWindowsExitProc(clientData)
-    ClientData clientData;		/* Not used. */
+    ClientData clientData;		/* tsdPtr when handler was created. */
 {
     TkDisplay *dispPtr, *nextPtr;
     Tcl_Interp *interp;
-    ThreadSpecificData *tsdPtr = (ThreadSpecificData *) 
-            Tcl_GetThreadData(&dataKey, sizeof(ThreadSpecificData));
+    ThreadSpecificData *tsdPtr = (ThreadSpecificData *) clientData;
 
     /*
      * Finish destroying any windows that are in a
