@@ -11,7 +11,7 @@
 #	files by clicking on the file icons or by entering a filename
 #	in the "Filename:" entry.
 #
-# RCS: @(#) $Id: tkfbox.tcl,v 1.38 2003/02/21 14:13:26 dkf Exp $
+# RCS: @(#) $Id: tkfbox.tcl,v 1.38.2.8 2005/04/12 20:33:35 hobbs Exp $
 #
 # Copyright (c) 1994-1998 Sun Microsystems, Inc.
 #
@@ -238,6 +238,12 @@ proc ::tk::IconList_Create {w} {
     set data(noScroll) 1
     set data(selection) {}
     set data(index,anchor) ""
+    set fg [option get $data(canvas) foreground Foreground]
+    if {$fg eq ""} {
+	set data(fill) black
+    } else {
+	set data(fill) $fg
+    }
 
     # Creates the event bindings.
     #
@@ -342,7 +348,7 @@ proc ::tk::IconList_Add {w image items} {
 	set iTag [$data(canvas) create image 0 0 -image $image -anchor nw \
 		-tags [list icon $data(numItems) item$data(numItems)]]
 	set tTag [$data(canvas) create text  0 0 -text  $text  -anchor nw \
-		-font $data(font) \
+		-font $data(font) -fill $data(fill) \
 		-tags [list text $data(numItems) item$data(numItems)]]
 	set rTag [$data(canvas) create rect  0 0 0 0 -fill "" -outline "" \
 		-tags [list rect $data(numItems) item$data(numItems)]]
@@ -767,7 +773,9 @@ proc ::tk::IconList_Reset {w} {
 
 namespace eval ::tk::dialog {}
 namespace eval ::tk::dialog::file {
-    namespace import ::tk::msgcat::*
+    namespace import -force ::tk::msgcat::*
+    set ::tk::dialog::file::showHiddenBtn 0
+    set ::tk::dialog::file::showHiddenVar 1
 }
 
 # ::tk::dialog::file:: --
@@ -798,7 +806,7 @@ proc ::tk::dialog::file:: {type args} {
     #
     if {![winfo exists $w]} {
 	::tk::dialog::file::Create $w TkFDialog
-    } elseif {[string compare [winfo class $w] TkFDialog]} {
+    } elseif {[winfo class $w] ne "TkFDialog"} {
 	destroy $w
 	::tk::dialog::file::Create $w TkFDialog
     } else {
@@ -807,13 +815,24 @@ proc ::tk::dialog::file:: {type args} {
 	set data(upBtn) $w.f1.up
 	set data(icons) $w.icons
 	set data(ent) $w.f2.ent
-	set data(typeMenuLab) $w.f2.lab
+	set data(typeMenuLab) $w.f2.lab2
 	set data(typeMenuBtn) $w.f2.menu
 	set data(typeMenu) $data(typeMenuBtn).m
 	set data(okBtn) $w.f2.ok
 	set data(cancelBtn) $w.f2.cancel
+	set data(hiddenBtn) $w.f2.hidden
 	::tk::dialog::file::SetSelectMode $w $data(-multiple)
     }
+    if {$::tk::dialog::file::showHiddenBtn} {
+	$data(hiddenBtn) configure -state normal
+	grid $data(hiddenBtn)
+    } else {
+	$data(hiddenBtn) configure -state disabled
+	grid remove $data(hiddenBtn)
+    }
+
+    # Make sure subseqent uses of this dialog are independent [Bug 845189]
+    catch {unset data(extUsed)}
 
     # Dialog boxes should be transient with respect to their parent,
     # so that they will always stay on top of their parent window.  However,
@@ -822,7 +841,7 @@ proc ::tk::dialog::file:: {type args} {
     # window, this can hang the entire application.  Therefore we only make
     # the dialog transient if the parent is viewable.
 
-    if {[winfo viewable [winfo toplevel $data(-parent)]] } {
+    if {[winfo viewable [winfo toplevel $data(-parent)]]} {
 	wm transient $w $data(-parent)
     }
 
@@ -1010,7 +1029,7 @@ static char updir_bits[] = {
     $data(upBtn) config -image $Priv(updirImage)
 
     $f1.menu config -takefocus 1 -highlightthickness 2
- 
+
     pack $data(upBtn) -side right -padx 4 -fill both
     pack $f1.lab -side left -padx 4 -fill both
     pack $f1.menu -expand yes -fill both -padx 4
@@ -1049,20 +1068,8 @@ static char updir_bits[] = {
 
     # Make the file types bits only if this is a File Dialog
     if { [string equal $class TkFDialog] } {
-	# The "File of types:" label needs to be grayed-out when
-	# -filetypes are not specified. The label widget does not support
-	# grayed-out text on monochrome displays. Therefore, we have to
-	# use a button widget to emulate a label widget (by setting its
-	# bindtags)
-
-	set data(typeMenuLab) [::tk::AmpWidget button $f2.lab2 \
-		-text $fTypeCaption  -anchor e  -bd [$f2.lab cget -bd] \
-		-highlightthickness [$f2.lab cget -highlightthickness] \
-		-relief [$f2.lab cget -relief] \
-		-padx [$f2.lab cget -padx] \
-		-pady [$f2.lab cget -pady]]
-	bindtags $data(typeMenuLab) [list $data(typeMenuLab) Label \
-		[winfo toplevel $data(typeMenuLab)] all]
+	set data(typeMenuLab) [::tk::AmpWidget label $f2.lab2 \
+		-text $fTypeCaption -anchor e -pady [$f2.lab cget -pady]]
 	set data(typeMenuBtn) [menubutton $f2.menu -indicatoron 1 \
 		-menu $f2.menu.m]
 	set data(typeMenu) [menu $data(typeMenuBtn).m -tearoff 0]
@@ -1072,12 +1079,30 @@ static char updir_bits[] = {
 		focus $data(typeMenuBtn)]
     }
 
+    # The hidden button is displayed when ::tk::dialog::file::showHiddenBtn
+    # is true.  Create it disabled so the binding doesn't trigger if it
+    # isn't shown.
+    if {$class eq "TkFDialog"} {
+	set text [mc "Show &Hidden Files and Directories"]
+    } else {
+	set text [mc "Show &Hidden Directories"]
+    }
+    set data(hiddenBtn) [::tk::AmpWidget checkbutton $f2.hidden \
+	    -text $text -anchor w -padx 3 -state disabled \
+	    -variable ::tk::dialog::file::showHiddenVar \
+	    -command [list ::tk::dialog::file::UpdateWhenIdle $w]]
+
     # the okBtn is created after the typeMenu so that the keyboard traversal
-    # is in the right order
+    # is in the right order, and add binding so that we find out when the
+    # dialog is destroyed by the user (added here instead of to the overall
+    # window so no confusion about how much <Destroy> gets called; exactly
+    # once will do). [Bug 987169]
+
     set data(okBtn)     [::tk::AmpWidget button $f2.ok \
-	    -text "[mc "&OK"]"     -default active -pady 3]
+	    -text [mc "&OK"]     -default active -pady 3]
+    bind $data(okBtn) <Destroy> [list ::tk::dialog::file::Destroyed $w]
     set data(cancelBtn) [::tk::AmpWidget button $f2.cancel \
-	    -text "[mc "&Cancel"]" -default normal -pady 3]
+	    -text [mc "&Cancel"] -default normal -pady 3]
 
     # grid the widgets in f2
     #
@@ -1087,8 +1112,9 @@ static char updir_bits[] = {
 	grid $data(typeMenuLab) $data(typeMenuBtn) $data(cancelBtn) \
 		-padx 4 -sticky ew
 	grid configure $data(typeMenuBtn) -padx 0
+	grid $data(hiddenBtn) -columnspan 2 -padx 4 -sticky ew
     } else {
-	grid x x $data(cancelBtn) -padx 4 -sticky ew
+	grid $data(hiddenBtn) - $data(cancelBtn) -padx 4 -sticky ew
     }
     grid columnconfigure $f2 1 -weight 1
 
@@ -1106,9 +1132,9 @@ static char updir_bits[] = {
     $data(cancelBtn) config -command [list ::tk::dialog::file::CancelCmd $w]
     bind $w <KeyPress-Escape> [list tk::ButtonInvoke $data(cancelBtn)]
     bind $w <Alt-Key> [list tk::AltKeyInDialog $w %A]
+
     # Set up event handlers specific to File or Directory Dialogs
     #
-
     if { [string equal $class TkFDialog] } {
 	bind $data(ent) <Return> [list ::tk::dialog::file::ActivateEnt $w]
 	$data(okBtn)     config -command [list ::tk::dialog::file::OkCmd $w]
@@ -1124,6 +1150,7 @@ static char updir_bits[] = {
 	bind $w <Alt-s> [list focus $data(ent)]
 	bind $w <Alt-o> [list tk::ButtonInvoke $data(okBtn)]
     }
+    bind $w <Alt-h> [list $data(hiddenBtn) invoke]
 
     # Build the focus group for all the entries
     #
@@ -1189,8 +1216,7 @@ proc ::tk::dialog::file::Update {w} {
 	return
     }
     set class [winfo class $w]
-    if { [string compare $class TkFDialog] && \
-	    [string compare $class TkChooseDir] } {
+    if {($class ne "TkFDialog") && ($class ne "TkChooseDir")} {
 	return
     }
 
@@ -1219,9 +1245,8 @@ rSASvJTGhnhcV3EJlo3kh53ltF5nAhQAOw==}]
 	# should have been checked before ::tk::dialog::file::Update is called, so
 	# we normally won't come to here. Anyways, give an error and abort
 	# action.
-	tk_messageBox -type ok -parent $w -message \
-	    "[mc "Cannot change to the directory \"%1\$s\".\nPermission denied." $data(selectPath)]"\
-	    -icon warning
+	tk_messageBox -type ok -parent $w -icon warning -message \
+	    [mc "Cannot change to the directory \"%1\$s\".\nPermission denied." $data(selectPath)]
 	cd $appPWD
 	return
     }
@@ -1234,47 +1259,38 @@ rSASvJTGhnhcV3EJlo3kh53ltF5nAhQAOw==}]
     $data(ent) config -cursor watch
     $w         config -cursor watch
     update idletasks
-    
+
     ::tk::IconList_DeleteAll $data(icons)
 
+    set showHidden $::tk::dialog::file::showHiddenVar
+
     # Make the dir list
-    #
-    set completeFileList [lsort -dictionary -unique [glob -nocomplain .* *]]
+    # Using -directory [pwd] is better in some VFS cases.
+    set cmd [list glob -tails -directory [pwd] -type d -nocomplain *]
+    if {$showHidden} { lappend cmd .* }
+    set dirs [lsort -dictionary -unique [eval $cmd]]
     set dirList {}
-    foreach f $completeFileList {
-	if {[string equal $f .]} {
+    foreach d $dirs {
+	if {$d eq "." || $d eq ".."} {
 	    continue
 	}
-	if {[string equal $f ..]} {
-	    continue
-	}
-	if {[file isdir ./$f]} {
-	    lappend dirList $f
-	}
+	lappend dirList $d
     }
     ::tk::IconList_Add $data(icons) $folder $dirList
-    if { [string equal $class TkFDialog] } {
-	# Make the file list if this is a File Dialog
+
+    if {$class eq "TkFDialog"} {
+	# Make the file list if this is a File Dialog, selecting all
+	# but 'd'irectory type files.
 	#
+	set cmd [list glob -tails -directory [pwd] \
+		     -type {f b c l p s} -nocomplain]
 	if {[string equal $data(filter) *]} {
-	    set files $completeFileList
+	    lappend cmd *
+	    if {$showHidden} { lappend cmd .* }
 	} else {
-	    set files {}
-	    foreach f $completeFileList {
-		foreach pat $data(filter) {
-		    if { [string match $pat $f] } {
-			lappend files $f
-			break
-		    }
-		}
-	    }
-	}
-	set fileList {}
-	foreach f $files {
-	    if {![file isdir ./$f]} {
-		lappend fileList $f
-	    }
+	    eval [list lappend cmd] $data(filter)
 	}
+	set fileList [lsort -dictionary -unique [eval $cmd]]
 	::tk::IconList_Add $data(icons) $file $fileList
     }
 
@@ -1519,10 +1535,17 @@ proc ::tk::dialog::file::ActivateEnt {w} {
 	# names as a true list, watching out for a single file with a
 	# space in the name.  Thus we query the IconList directly.
 
+	set selIcos [::tk::IconList_Curselection $data(icons)]
 	set data(selectFile) ""
-	foreach item [::tk::IconList_Curselection $data(icons)] {
-	    ::tk::dialog::file::VerifyFileName $w \
+	if {[llength $selIcos] == 0 && $text ne ""} {
+	    # This assumes the user typed something in without selecting
+	    # files - so assume they only type in a single filename.
+	    ::tk::dialog::file::VerifyFileName $w $text
+	} else {
+	    foreach item $selIcos {
+		::tk::dialog::file::VerifyFileName $w \
 		    [::tk::IconList_Get $data(icons) $item]
+	    }
 	}
     } else {
 	::tk::dialog::file::VerifyFileName $w $text
@@ -1659,6 +1682,16 @@ proc ::tk::dialog::file::CancelCmd {w} {
     upvar ::tk::dialog::file::[winfo name $w] data
     variable ::tk::Priv
 
+    bind $data(okBtn) <Destroy> {}
+    set Priv(selectFilePath) ""
+}
+
+# Gets called when user destroys the dialog directly [Bug 987169]
+#
+proc ::tk::dialog::file::Destroyed {w} {
+    upvar ::tk::dialog::file::[winfo name $w] data
+    variable ::tk::Priv
+
     set Priv(selectFilePath) ""
 }
 
@@ -1780,5 +1813,6 @@ proc ::tk::dialog::file::Done {w {selectFilePath ""}} {
 	    }
 	}
     }
+    bind $data(okBtn) <Destroy> {}
     set Priv(selectFilePath) $selectFilePath
 }
