@@ -10,7 +10,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tkMacOSXButton.c,v 1.2.2.5 2004/02/16 00:42:34 wolfsuit Exp $
+ * RCS: @(#) $Id: tkMacOSXButton.c,v 1.2.2.9 2005/05/14 20:53:31 das Exp $
  */
 
 #include "tkButton.h"
@@ -197,9 +197,6 @@ TkpDisplayButton(
     MacButton *macButtonPtr = (MacButton *)clientData;
     TkButton  *butPtr       = (TkButton *) clientData;
     Tk_Window tkwin         = butPtr->tkwin;
-    int x = 0;                  /* Initialization only needed to stop
-                                 * compiler warning. */
-    int y;
     int width, height, fullWidth, fullHeight;
     int textXOffset, textYOffset;
     int haveImage = 0, haveText = 0;
@@ -234,11 +231,15 @@ TkpDisplayButton(
     }
    
     /* 
-     * set up clipping region 
+     * set up clipping region.  Make sure the we are using the port
+     * for this button, or we will set the wrong window's clip. 
      */
     
+    destPort = TkMacOSXGetDrawablePort(pixmap);
+    SetGWorld(destPort, NULL);
     TkMacOSXSetUpClippingRgn(pixmap);
 
+
     /*
      * See the comment in UpdateControlColors as to why we use the 
      * highlightbackground for the border of Macintosh buttons.
@@ -262,13 +263,6 @@ TkpDisplayButton(
 
     if (macButtonPtr->usingControl) {
         borderWidth = 0;
-        /*
-         * This part uses Macintosh rather than Tk calls to draw
-         * to the screen.  Make sure the ports etc. are set correctly.
-         */
-         
-        destPort = TkMacOSXGetDrawablePort(pixmap);
-        SetGWorld(destPort, NULL);
         TkMacOSXDrawControl(macButtonPtr, destPort, dpPtr->gc, pixmap);
     } else {
        if (wasUsingControl && macButtonPtr->userPane) {
@@ -304,6 +298,8 @@ TkpDisplayButton(
 
         haveText = (butPtr->textWidth != 0 && butPtr->textHeight != 0);
         if (butPtr->compound != COMPOUND_NONE && haveImage && haveText) {
+            int x;
+            int y;
             textXOffset = 0;
             textYOffset = 0;
             fullWidth = 0;
@@ -407,6 +403,9 @@ TkpDisplayButton(
             y += fullHeight/2;
         } else {
             if (haveImage) {
+                int x = 0;
+                int y;
+
                 TkComputeAnchor(butPtr->anchor, tkwin, 0, 0,
                         butPtr->indicatorSpace + width, height, &x, &y);
                 x += butPtr->indicatorSpace;
@@ -440,19 +439,16 @@ TkpDisplayButton(
                     XSetClipOrigin(butPtr->display, dpPtr->gc, 0, 0);
                 }
                 y += height/2;
-            } else {
+            } else if (macButtonPtr->useTkText) {
+                int x = 0;
+                int y;
                 TkComputeAnchor(butPtr->anchor, tkwin, butPtr->padX,
                         butPtr->padY,
                         butPtr->indicatorSpace + butPtr->textWidth,
-                        butPtr->textHeight, &x, &y);
-                
-                x += butPtr->indicatorSpace;
-                
-                if (macButtonPtr->useTkText) {
-                    Tk_DrawTextLayout(butPtr->display, pixmap, dpPtr->gc,
+                        butPtr->textHeight, &x, &y);                
+                x += butPtr->indicatorSpace;                
+                Tk_DrawTextLayout(butPtr->display, pixmap, dpPtr->gc,
                         butPtr->textLayout, x, y, 0, -1);
-                }
-                y += butPtr->textHeight/2;
             }
         }
     }
@@ -822,13 +818,12 @@ TkMacOSXInitControl (
      */
 
     initiallyVisible = false;
-    initialValue = kControlSupportsEmbedding|   
-        kControlHasSpecialBackground;
-    minValue = 0;
-    maxValue = 1;
-    procID = kControlUserPaneProc;
+    initialValue     = kControlSupportsEmbedding|kControlHasSpecialBackground;
+    minValue         = 0;
+    maxValue         = 1;
+    procID           = kControlUserPaneProc;
     controlReference = (SInt32)mbPtr;
-    mbPtr->userPane = NewControl(mbPtr->windowRef,
+    mbPtr->userPane  = NewControl(mbPtr->windowRef,
         paneRect, "\p",
         initiallyVisible,
         initialValue,
@@ -842,7 +837,7 @@ TkMacOSXInitControl (
         return 1;
     }
     
-    if ((status=EmbedControl(mbPtr->userPane,rootControl)) != noErr) {
+    if ((status = EmbedControl(mbPtr->userPane,rootControl)) != noErr) {
         fprintf(stderr,"Failed to embed user pane control %d\n", status);
         return 1;
     }
@@ -912,22 +907,22 @@ TkMacOSXDrawControl(
 
     winPtr = (TkWindow *)butPtr->tkwin;
    
-    paneRect.left = winPtr->privatePtr->xOff;
-    paneRect.top = winPtr->privatePtr->yOff;
-    paneRect.right = paneRect.left + Tk_Width(butPtr->tkwin);
+    paneRect.left   = winPtr->privatePtr->xOff;
+    paneRect.top    = winPtr->privatePtr->yOff;
+    paneRect.right  = paneRect.left + Tk_Width(butPtr->tkwin);
     paneRect.bottom = paneRect.top + Tk_Height(butPtr->tkwin);
 
     cntrRect = paneRect;
 
 /*
-    cntrRect.left += butPtr->inset;
-    cntrRect.top += butPtr->inset;
-    cntrRect.right -= butPtr->inset;
+    cntrRect.left   += butPtr->inset;
+    cntrRect.top    += butPtr->inset;
+    cntrRect.right  -= butPtr->inset;
     cntrRect.bottom -= butPtr->inset;
 */
-    cntrRect.left += DEF_INSET_LEFT;
-    cntrRect.top += DEF_INSET_TOP;
-    cntrRect.right -= DEF_INSET_RIGHT;
+    cntrRect.left   += DEF_INSET_LEFT;
+    cntrRect.top    += DEF_INSET_TOP;
+    cntrRect.right  -= DEF_INSET_RIGHT;
     cntrRect.bottom -= DEF_INSET_BOTTOM;
 
     /* 
@@ -967,13 +962,19 @@ TkMacOSXDrawControl(
         Tk_Font    font;
         int        len;
         
-        len = TkFontGetFirstTextLayout(butPtr->textLayout, 
-                &font, controlTitle);
-        controlTitle[len] = 0;
+        if (((mbPtr->info.image == NULL) && (mbPtr->info.bitmap == None))
+               || (mbPtr->info.compound != COMPOUND_NONE)) {
+            len = TkFontGetFirstTextLayout(butPtr->textLayout, 
+                    &font, (char*) controlTitle);
+            controlTitle[len] = 0;
+        } else {
+            len = 0;
+            controlTitle[0] = 0;
+        }
         if (bcmp(mbPtr->controlTitle, controlTitle, len+1)) {
             CFStringRef cf;    	    
             cf = CFStringCreateWithCString(NULL,
-                  controlTitle, kCFStringEncodingUTF8);
+                  (char*) controlTitle, kCFStringEncodingUTF8);
             if (cf != NULL) {
             SetControlTitleWithCFString(mbPtr->control, cf);
             CFRelease(cf);
@@ -1041,12 +1042,12 @@ TkMacOSXDrawControl(
     }
 
     if (mbPtr->flags&2) {
-        ShowControl(mbPtr->control);
         ShowControl(mbPtr->userPane);
+        ShowControl(mbPtr->control);
         mbPtr->flags ^= 2;
     } else {
-        Draw1Control(mbPtr->userPane);
         SetControlVisibility(mbPtr->control, true, true);
+        Draw1Control(mbPtr->userPane);
     }
  
     if (mbPtr->params.isBevel) {
@@ -1140,7 +1141,7 @@ SetupBevelButton(
     if ((err = SetControlData(controlHandle, kControlButtonPart,
             kControlBevelButtonContentTag,
             sizeof(ControlButtonContentInfo),
-            (char *) &mbPtr->bevelButtonContent)) != noErr ) {
+            (char *) &mbPtr->bevelButtonContent)) != noErr) {
         fprintf(stderr,
                 "SetControlData BevelButtonContent failed, %d\n", err );
     }
@@ -1168,7 +1169,7 @@ SetupBevelButton(
     if ((err = SetControlData(controlHandle, kControlButtonPart,
             kControlBevelButtonGraphicAlignTag,
             sizeof(ControlButtonGraphicAlignment),
-            (char *) &theAlignment)) != noErr ) {
+            (char *) &theAlignment)) != noErr) {
         fprintf(stderr,
                 "SetControlData BevelButtonGraphicAlign failed, %d\n", err );
     }
@@ -1188,7 +1189,7 @@ SetupBevelButton(
         if ((err = SetControlData(controlHandle, kControlButtonPart,
                 kControlBevelButtonTextPlaceTag,
                 sizeof(ControlButtonTextPlacement),
-                (char *) &thePlacement)) != noErr ) {
+                (char *) &thePlacement)) != noErr) {
             fprintf(stderr,
                     "SetControlData BevelButtonTextPlace failed, %d\n", err );
         }
