@@ -9,7 +9,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tkMacOSXDialog.c,v 1.4.2.2 2004/02/16 00:42:34 wolfsuit Exp $
+ * RCS: @(#) $Id: tkMacOSXDialog.c,v 1.4.2.4 2005/05/14 20:53:31 das Exp $
  */
 #include <Carbon/Carbon.h>
 
@@ -81,7 +81,7 @@ pascal void             OpenEventProc(NavEventCallbackMessage callBackSelector,
 static void             InitFileDialogs();
 static int              NavServicesGetFile(Tcl_Interp *interp, OpenFileData *ofd,
                             AEDesc *initialDescPtr,
-                            unsigned char *initialFile, AEDescList *selectDescPtr,
+                            char *initialFile, AEDescList *selectDescPtr,
                             CFStringRef title, CFStringRef message, int multiple, int isOpen);
 static int              HandleInitialDirectory (Tcl_Interp *interp,
                                                 char *initialFile, char *initialDir,
@@ -214,7 +214,7 @@ Tk_ChooseColorObjCmd(
     cpinfo.pickerType = 0L;
     cpinfo.eventProc = NULL;
     cpinfo.colorProc = NULL;
-    cpinfo.colorProcData = NULL;
+    cpinfo.colorProcData = 0;
     
     /* This doesn't seem to actually set the title! */
     Tcl_UtfToExternal(NULL, NULL, title, -1, 0, NULL, 
@@ -338,7 +338,7 @@ Tk_GetOpenFileObjCmd(
                 break;
             case OPEN_MESSAGE:
                 choice = Tcl_GetStringFromObj(objv[i + 1], &choiceLen);
-                message = CFStringCreateWithBytes(NULL, choice, choiceLen, 
+                message = CFStringCreateWithBytes(NULL, (unsigned char*) choice, choiceLen, 
                         kCFStringEncodingUTF8, false);
                 break;
             case OPEN_MULTIPLE:
@@ -358,7 +358,7 @@ Tk_GetOpenFileObjCmd(
                 break;
             case OPEN_TITLE:
                 choice = Tcl_GetStringFromObj(objv[i + 1], &choiceLen);
-                title = CFStringCreateWithBytes(NULL, choice, choiceLen, 
+                title = CFStringCreateWithBytes(NULL, (unsigned char*) choice, choiceLen, 
                         kCFStringEncodingUTF8, false);
                 break;
         }
@@ -415,7 +415,7 @@ Tk_GetSaveFileObjCmd(
     Tcl_Obj *CONST objv[])     /* Argument objects. */
 {
     int i, result;
-    Str255 initialFile;
+    char *initialFile = NULL;
     Tk_Window parent;
     AEDesc initialDesc = {typeNull, NULL};
     AEDesc *initialPtr = NULL;
@@ -437,7 +437,6 @@ Tk_GetSaveFileObjCmd(
     
     result = TCL_ERROR;    
     parent = (Tk_Window) clientData;    
-    StrLength(initialFile) = 0;
     title = NULL;
     message = NULL;   
 
@@ -445,8 +444,6 @@ Tk_GetSaveFileObjCmd(
         char *choice;
         int index, choiceLen;
         char *string;
-        Tcl_DString ds;
-        int srcRead, dstWrote;
 
         if (Tcl_GetIndexFromObj(interp, objv[i], saveOptionStrings, "option",
                 TCL_EXACT, &index) != TCL_OK) {
@@ -473,20 +470,11 @@ Tk_GetSaveFileObjCmd(
                 }
                 break;
             case SAVE_INITFILE:
-                choice = Tcl_GetStringFromObj(objv[i + 1], &choiceLen);
-                if (Tcl_TranslateFileName(interp, choice, &ds) == NULL) {
-                    result = TCL_ERROR;
-                    goto end;
-                }
-                Tcl_UtfToExternal(NULL, TkMacOSXCarbonEncoding, Tcl_DStringValue(&ds), 
-                        Tcl_DStringLength(&ds), 0, NULL, 
-                        StrBody(initialFile), 255, &srcRead, &dstWrote, NULL);
-                StrLength(initialFile) = (unsigned char) dstWrote;
-                Tcl_DStringFree(&ds);            
+                initialFile = Tcl_GetStringFromObj(objv[i + 1], NULL);
                 break;
             case SAVE_MESSAGE:
                 choice = Tcl_GetStringFromObj(objv[i + 1], &choiceLen);
-                message = CFStringCreateWithBytes(NULL, choice, choiceLen, 
+                message = CFStringCreateWithBytes(NULL, (unsigned char*) choice, choiceLen, 
                         kCFStringEncodingUTF8, false);
                 break;
             case SAVE_PARENT:
@@ -499,7 +487,7 @@ Tk_GetSaveFileObjCmd(
                 break;
             case SAVE_TITLE:
                 choice = Tcl_GetStringFromObj(objv[i + 1], &choiceLen);
-                title = CFStringCreateWithBytes(NULL, choice, choiceLen, 
+                title = CFStringCreateWithBytes(NULL, (unsigned char*) choice, choiceLen, 
                         kCFStringEncodingUTF8, false);
                 break;
         }
@@ -607,7 +595,7 @@ Tk_ChooseDirectoryObjCmd(clientData, interp, objc, objv)
                 break;
             case CHOOSE_MESSAGE:
                 choice = Tcl_GetStringFromObj(objv[i + 1], &choiceLen);
-                message = CFStringCreateWithBytes(NULL, choice, choiceLen, 
+                message = CFStringCreateWithBytes(NULL, (unsigned char*) choice, choiceLen, 
                         kCFStringEncodingUTF8, false);
                 break;
             case CHOOSE_PARENT:
@@ -620,7 +608,7 @@ Tk_ChooseDirectoryObjCmd(clientData, interp, objc, objv)
                 break;
             case CHOOSE_TITLE:
                 choice = Tcl_GetStringFromObj(objv[i + 1], &choiceLen);
-                title = CFStringCreateWithBytes(NULL, choice, choiceLen, 
+                title = CFStringCreateWithBytes(NULL, (unsigned char*) choice, choiceLen, 
                         kCFStringEncodingUTF8, false);
                 break;
         }
@@ -669,7 +657,7 @@ HandleInitialDirectory (
             return TCL_ERROR;
         }
 
-        err = FSPathMakeRef(dirName,
+        err = FSPathMakeRef((unsigned char*) dirName,
                 dirRef, &isDirectory);     
 
         if (err != noErr) {
@@ -703,7 +691,7 @@ HandleInitialDirectory (
 
         AECreateList(NULL, 0, false, selectDescPtr);
 
-        err = FSPathMakeRef(namePtr, &fileRef, &isDirectory);
+        err = FSPathMakeRef((unsigned char*) namePtr, &fileRef, &isDirectory);
         if (err != noErr) {
             Tcl_AppendResult(interp, "bad initialfile \"", initialFile,
                     "\" file does not exist.", NULL);
@@ -734,7 +722,7 @@ NavServicesGetFile(
     Tcl_Interp *interp,
     OpenFileData *ofdPtr,
     AEDesc *initialDescPtr,
-    unsigned char *initialFile,
+    char *initialFile,
     AEDescList *selectDescPtr,
     CFStringRef title,
     CFStringRef message,
@@ -748,9 +736,7 @@ NavServicesGetFile(
     OSErr err;
     Tcl_Obj *theResult = NULL;
     int result;
-    TextEncoding encoding;
 
-    encoding = GetApplicationTextEncoding();
     err = NavGetDefaultDialogCreationOptions(&diagOptions);
     if (err!=noErr) {
         return TCL_ERROR;
@@ -784,7 +770,7 @@ NavServicesGetFile(
         for (filterPtr = ofdPtr->fl.filters; filterPtr != NULL; 
                 filterPtr = filterPtr->next, index++) {
             menuItemNames[index] = CFStringCreateWithCString(NULL, 
-                    filterPtr->name, encoding);
+                    filterPtr->name, kCFStringEncodingUTF8);
         }
         diagOptions.popupExtension = CFArrayCreate(NULL, 
                 (const void **) menuItemNames, ofdPtr->fl.numFilters, NULL);
@@ -800,12 +786,12 @@ NavServicesGetFile(
     
     diagOptions.optionFlags += kNavSupportPackages;
     
-    diagOptions.clientName = CFStringCreateWithCString(NULL, "Wish", encoding);
+    diagOptions.clientName = CFStringCreateWithCString(NULL, "Wish", kCFStringEncodingUTF8);
     diagOptions.message = message;
     diagOptions.windowTitle = title;
-    if ((initialFile != NULL) && (initialFile[0] != 0)) {
-        diagOptions.saveFileName = CFStringCreateWithPascalString(NULL,
-                initialFile, encoding);
+    if (initialFile) {
+        diagOptions.saveFileName = CFStringCreateWithCString(NULL,
+                initialFile, kCFStringEncodingUTF8);
     } else {
         diagOptions.saveFileName = NULL;
     }
@@ -882,7 +868,6 @@ NavServicesGetFile(
     if (theReply.validRecord && err == noErr) {
         AEDesc resultDesc;
         long count;
-        Tcl_DString fileName;
         FSRef  fsRef;
         char   pathPtr[1024];
         int    pathValid = 0;
@@ -898,7 +883,7 @@ NavServicesGetFile(
                             != noErr ) {
                         fprintf(stderr,"AEGetDescData failed %d\n", err );
                     } else {
-                        if (err = FSRefMakePath(&fsRef, pathPtr, 1024) ) {
+                        if (err = FSRefMakePath(&fsRef, (unsigned char*) pathPtr, 1024) ) {
                             fprintf(stderr,"FSRefMakePath failed, %d\n", err );
                         } else {
                             if (isOpen == SAVE_FILE) {
@@ -906,10 +891,14 @@ NavServicesGetFile(
                                 char saveName [1024];
                                 if (saveNameRef = NavDialogGetSaveFileName(dialogRef)) {
                                     if (CFStringGetCString(saveNameRef, saveName, 
-                                            1024, encoding)) {
-                                        strcat(pathPtr, "/");
-                                        strcat(pathPtr, saveName);
-                                        pathValid = 1;
+                                            1024, kCFStringEncodingUTF8)) {
+                                        if (strlen(pathPtr) + strlen(saveName) < 1023) {
+                                            strcat(pathPtr, "/");
+                                            strcat(pathPtr, saveName);
+                                            pathValid = 1;
+                                        } else {
+                                            fprintf(stderr, "Path name too long\n");                                        
+                                        }
                                     } else {
                                         fprintf(stderr, "CFStringGetCString failed\n");
                                     }
@@ -920,21 +909,12 @@ NavServicesGetFile(
                                 pathValid = 1;
                             }
                             if (pathValid) {
-                                /* 
-                                 * Tested this and NULL=utf-8 encoding is
-                                 * good here
-                                 */
-                                Tcl_ExternalToUtfDString(NULL, pathPtr, -1, 
-							 &fileName);
                                 if (multiple) {
                                     Tcl_ListObjAppendElement(interp, theResult, 
-                                        Tcl_NewStringObj(Tcl_DStringValue(&fileName), 
-                                        Tcl_DStringLength(&fileName)));
+                                        Tcl_NewStringObj(pathPtr, -1));
                                 } else {
-                                    Tcl_SetStringObj(theResult, Tcl_DStringValue(&fileName), 
-                                        Tcl_DStringLength(&fileName));
+                                    Tcl_SetStringObj(theResult, pathPtr, -1);
                                 }
-                                Tcl_DStringFree(&fileName);
                             }
                         }
                     }
@@ -1021,9 +1001,9 @@ OpenFileFilterProc(
                         int len;
                         fileNamePtr = (((FSSpec *) *theItem->dataHandle)->name);
                         len = fileNamePtr[0];
-                        strncpy(fileName, fileNamePtr + 1, len);
+                        strncpy(fileName, (char*) fileNamePtr + 1, len);
                         fileName[len] = '\0';
-                        fileNamePtr = fileName;
+                        fileNamePtr = (unsigned char*) fileName;
 
                     } else if (theItem->descriptorType = typeFSRef) {
                         OSStatus err;
@@ -1037,7 +1017,7 @@ OpenFileFilterProc(
                                     (Tcl_UniChar *) uniFileName.unicode, 
                                     uniFileName.length,
                                     &fileNameDString);
-                            fileNamePtr = Tcl_DStringValue(&fileNameDString);
+                            fileNamePtr = (unsigned char*) Tcl_DStringValue(&fileNameDString);
                         } else {
                             fileNamePtr = NULL;
                         }
@@ -1182,7 +1162,7 @@ MatchOneType(
                  * have "." in it
                  */
 
-                for (q = fileNamePtr; *q; q++) {
+                for (q = (char*) fileNamePtr; *q; q++) {
                     if (*q == '.') {
                         goto glob_unmatched;
                     }
@@ -1190,7 +1170,7 @@ MatchOneType(
                 goto glob_matched;
             }
         
-            if (Tcl_StringMatch(fileNamePtr, ext)) {
+            if (Tcl_StringMatch((char*) fileNamePtr, ext)) {
                 goto glob_matched;
             } else {
                 goto glob_unmatched;
@@ -1316,9 +1296,9 @@ Tk_MessageBoxObjCmd(
     int                     result = TCL_OK;
     
     static CONST char *movableAlertStrings[] = {
-	"-default", /* "-finemessage", */ "-icon", 
+	"-default", /* "-detail", */ "-icon", 
 	"-message", "-parent", 
-	"-title", "-type", 	
+	"-title", "-type",
 	(char *)NULL
     };
     static CONST char *movableTypeStrings[] = {
@@ -1337,7 +1317,7 @@ Tk_MessageBoxObjCmd(
 	(char *)NULL
     };
     enum movableAlertOptions {
-	ALERT_DEFAULT, /* ALERT_FINEMESSAGE, */ ALERT_ICON,
+	ALERT_DEFAULT, /* ALERT_DETAIL, */ ALERT_ICON,
 	ALERT_MESSAGE, ALERT_PARENT,
 	ALERT_TITLE, ALERT_TYPE
     };
@@ -1422,7 +1402,7 @@ Tk_MessageBoxObjCmd(
 	    indexDefaultOption = i;
 	    break;
 	    
-/*	    case ALERT_FINEMESSAGE:
+/*	    case ALERT_DETAIL:
 	    str = Tcl_GetStringFromObj( objv[i + 1], NULL );
 	    finemessageTextCF = CFStringCreateWithCString( NULL, str, kCFStringEncodingUTF8 );
 	    break;
