@@ -11,11 +11,12 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tkMacOSXScrlbr.c,v 1.5 2002/11/20 05:21:08 wolfsuit Exp $
+ * RCS: @(#) $Id: tkMacOSXScrlbr.c,v 1.5.2.4 2005/06/03 00:53:29 wolfsuit Exp $
  */
 
 #include "tkScrollbar.h"
 #include "tkMacOSXInt.h"
+#include "tclInt.h"
 
 #include <Carbon/Carbon.h>
 
@@ -42,8 +43,8 @@ enum {
 
 #define MIN_SLIDER_LENGTH       5
 #define MIN_SCROLLBAR_VALUE     0
-#define MAX_SCROLLBAR_VALUE     1000
-#define MAX_SCROLLBAR_DVALUE    1000.0
+#define SCROLLBAR_SCALING_VALUE     100000
+#define SCROLLBAR_SCALING_DVALUE    100000.0
 
 /*
  * Declaration of Windows specific scrollbar structure.
@@ -95,8 +96,14 @@ typedef struct MacScrollbar {
  */
 static ControlActionUPP scrollActionProc = NULL; /* Pointer to func. */
 static ThumbActionUPP thumbActionProc = NULL;    /* Pointer to func. */
-static TkScrollbar *activeScrollPtr = NULL;        /* Non-null when in thumb */
+static TkScrollbar *activeScrollPtr = NULL;      /* Non-null when in thumb */
                                                  /* proc. */
+static Point mouseDownPoint;	/* Used when activeScrollPtr is non-NULL   */
+				/* to store the coordinates where the      */
+				/* mouse was first pressed to begin        */
+				/* dragging the thumb, because             */
+				/* ThumbActionProc can't take any args.    */
+						 
 /*
  * Forward declarations for procedures defined later in this file:
  */
@@ -237,9 +244,9 @@ TkpDisplayScrollbar(
 
     if (macScrollPtr->sbHandle == NULL) {
         Rect r;
-        SInt16 initialValue;
-        SInt16 minValue;
-        SInt16 maxValue;
+        SInt32 initialValue;
+        SInt32 minValue;
+        SInt32 maxValue;
         SInt16 procID;
         WindowRef frontNonFloating;
         
@@ -247,7 +254,7 @@ TkpDisplayScrollbar(
         r.right = r.bottom = 1;
 
         minValue = MIN_SCROLLBAR_VALUE;
-        maxValue = MAX_SCROLLBAR_VALUE;
+        maxValue = SCROLLBAR_SCALING_VALUE;
         initialValue = (minValue + maxValue)/2;
         procID = kControlScrollBarLiveProc;
 
@@ -578,8 +585,8 @@ ThumbActionProc()
     register TkScrollbar *scrollPtr = activeScrollPtr;
     register MacScrollbar *macScrollPtr = (MacScrollbar *) activeScrollPtr;
     Tcl_DString cmdString;
-    int origValue, trackBarPin;
-    double thumbWidth, newFirstFraction, trackBarSize;
+    int origValue;
+    double thumbWidth, oldFirstFraction, newFirstFraction, trackBarSize;
     char valueString[40];
     Point currentPoint = { 0, 0 };
     Rect trackRect;
@@ -603,29 +610,35 @@ ThumbActionProc()
      * Note: the arrowLength is equal to the thumb width of a Mac scrollbar.
      */
 
-    origValue = GetControlValue(macScrollPtr->sbHandle);
+    origValue = GetControl32BitValue(macScrollPtr->sbHandle);
     GetControlBounds(macScrollPtr->sbHandle, &trackRect);
+    
+    thumbWidth = scrollPtr->lastFraction - scrollPtr->firstFraction;
+    
     if (scrollPtr->vertical == true) {
         trackBarSize = (double) (trackRect.bottom - trackRect.top
-            - (scrollPtr->arrowLength * 3));
-        trackBarPin = trackRect.top + scrollPtr->arrowLength
-            + (scrollPtr->arrowLength / 2);
-        InsetRect(&trackRect, -25, -113);
+                - (scrollPtr->arrowLength * 3));
+        InsetRect(&trackRect, -50, -226);
         
     } else {
         trackBarSize = (double) (trackRect.right - trackRect.left
-            - (scrollPtr->arrowLength * 3));
-        trackBarPin = trackRect.left + scrollPtr->arrowLength
-            + (scrollPtr->arrowLength / 2);
-        InsetRect(&trackRect, -113, -25);
+                - (scrollPtr->arrowLength * 3));
+        InsetRect(&trackRect, -226, -50);
     }
 
     /*
      * Track the mouse while the button is held down.  If the mouse is moved,
      * we calculate the value that should be passed to the "command" part of
-     * the scrollbar.
+     * the scrollbar.  Since the mouse may move a distance too small to
+     * cause a change to the first fraction, each calculation must be done
+     * versus what the first fraction was when the mouse button was
+     * initially pressed.  Otherwise, moving the mouse too slowly will
+     * cause the calculated fraction delta to be zero and the scrollbar
+     * won't respond.
      */
 
+    oldFirstFraction = scrollPtr->firstFraction;
+
     do {
         err = TrackMouseLocationWithOptions(NULL,
                 kTrackMouseLocationOptionDontConsumeMouseUp,
@@ -634,28 +647,45 @@ ThumbActionProc()
                 NULL,
                 &trackingResult);
             
-        if ((err==noErr) 
+        if ((err == noErr) 
                 && ((trackingResult == kMouseTrackingMouseDragged)
                 || (trackingResult == kMouseTrackingMouseMoved))) {
-        /*
-         * Calculating this value is a little tricky.  We need to calculate a
-         * value for where the thumb would be in a Motif widget (variable
-         * thumb).  This value is what the "command" expects and is what will
-         * be resent to the scrollbar to update its value.
-         */
-            thumbWidth = scrollPtr->lastFraction - scrollPtr->firstFraction;
+
+	   /*
+            * Calculate where the scrollbar should move to, based on
+            * where the mouse button was pressed and where the scrollbar
+	    * initially was at that time.  Note that PtInRect() will
+	    * return false if currentPoint or trackRect are not in
+	    * is not in current graphics port, which may happen if any
+            * of the waiting idle events change the port (e.g. with
+	    * SetPort()) but fail to restore it before returning and the
+	    * scrollbar will lock in place.
+            */
+	    newFirstFraction = oldFirstFraction;
             if (PtInRect(currentPoint, &trackRect)) {
+		double pixDiff;
+		double fracDelta;
                 if (scrollPtr->vertical == true) {
-                    newFirstFraction =  (1.0 - thumbWidth) *
-                        ((double) (currentPoint.v - trackBarPin) / trackBarSize);
+		    pixDiff = (double)(currentPoint.v - mouseDownPoint.v);
                 } else {
-                    newFirstFraction =  (1.0 - thumbWidth) *
-                        ((double) (currentPoint.h - trackBarPin) / trackBarSize);
+		    pixDiff = (double)(currentPoint.h - mouseDownPoint.h);
                 }
-            } else {
-                newFirstFraction = ((double) origValue / MAX_SCROLLBAR_DVALUE)
-                   * (1.0 - thumbWidth);
-            }
+		fracDelta = pixDiff/(trackBarSize);
+		newFirstFraction += fracDelta;
+		if (newFirstFraction > 1.0) {
+		    newFirstFraction = 1.0;
+		} else if (newFirstFraction < 0.0) {
+		    newFirstFraction = 0.0;
+		}
+	    }
+            
+	    /*
+	     * Move the scrollbar thumb to the new first fraction given
+	     * its position when initially pressed and how far the mouse
+	     * has moved.  Process waiting idle tasks afterward to allow
+	     * for the display to update.
+	     */
+
             sprintf(valueString, "%g", newFirstFraction);
             Tcl_DStringSetLength(&cmdString, 0);
             Tcl_DStringAppend(&cmdString, scrollPtr->command,
@@ -666,14 +696,10 @@ ThumbActionProc()
             Tcl_Preserve((ClientData) interp);
             Tcl_GlobalEval(interp, cmdString.string);
             Tcl_Release((ClientData) interp);
-            Tcl_DStringSetLength(&cmdString, 0);
-            Tcl_DStringAppend(&cmdString, "update idletasks",
-                strlen("update idletasks"));
-            Tcl_Preserve((ClientData) interp);
-            Tcl_GlobalEval(interp, cmdString.string);
-            Tcl_Release((ClientData) interp);
+
+	    TclServiceIdle();
         }
-    } while ((err==noErr) && trackingResult!=kMouseTrackingMouseReleased );
+    } while ((err == noErr) && trackingResult != kMouseTrackingMouseReleased);
 
     Tcl_DStringFree(&cmdString);
     return;
@@ -763,10 +789,10 @@ ScrollbarBindProc(
     macScrollPtr->macFlags |= IN_MODAL_LOOP;
     
     if (eventPtr->type == ButtonPress) {
-            Point where;
-            Rect bounds;
-            int part, x, y, dummy;
-            unsigned int state;
+	Point where;
+	Rect bounds;
+	int part, x, y, dummy;
+	unsigned int state;
         CGrafPtr saveWorld;
         GDHandle saveDevice;
         GWorldPtr destPort;
@@ -783,8 +809,8 @@ ScrollbarBindProc(
         TkMacOSXSetUpClippingRgn(Tk_WindowId(scrollPtr->tkwin));
 
         TkMacOSXWinBounds((TkWindow *) scrollPtr->tkwin, &bounds);                
-            where.h = eventPtr->xbutton.x + bounds.left;
-            where.v = eventPtr->xbutton.y + bounds.top;
+	where.h = eventPtr->xbutton.x + bounds.left;
+	where.v = eventPtr->xbutton.y + bounds.top;
         part = TestControl(macScrollPtr->sbHandle, where);
         if (part == kControlIndicatorPart && scrollPtr->jump == false) {
             /*
@@ -794,6 +820,8 @@ ScrollbarBindProc(
              * so the callback may have access to it.
              */
             activeScrollPtr = scrollPtr;
+	    mouseDownPoint.h = where.h;
+	    mouseDownPoint.v = where.v;
             part = TrackControl(macScrollPtr->sbHandle, where,
                     (ControlActionUPP) thumbActionProc);
             activeScrollPtr = NULL;
@@ -816,8 +844,9 @@ ScrollbarBindProc(
                  */
                 thumbWidth = scrollPtr->lastFraction
                      - scrollPtr->firstFraction;
-                newFirstFraction = (1.0 - thumbWidth) *
-                    ((double) GetControlValue(macScrollPtr->sbHandle) / MAX_SCROLLBAR_DVALUE);
+		 newFirstFraction = (1.0 - thumbWidth) *
+		((double) GetControl32BitValue(macScrollPtr->sbHandle) / SCROLLBAR_SCALING_DVALUE);
+
                 sprintf(valueString, "%g", newFirstFraction);
 
                 Tcl_DStringInit(&cmdString);
@@ -931,8 +960,8 @@ UpdateControlValues(
     Tk_Window tkwin = scrollPtr->tkwin;
     MacDrawable * macDraw = (MacDrawable *) Tk_WindowId(scrollPtr->tkwin);
     WindowRef windowRef  = GetControlOwner(macScrollPtr->sbHandle);
-    double middle;
-    SInt32 viewSize;
+    double dViewSize;
+    SInt32 viewSize, controlMax, controlValue;
     int flushRight = false;
     int flushBottom = false;
     Rect contrlRect, portRect;
@@ -1066,27 +1095,50 @@ UpdateControlValues(
     /*
      * Given the Tk parameters for the fractions of the start and
      * end of the thumb, the following calculation determines the
-     * location for the fixed sized Macintosh thumb.
+     * location for the Macintosh thumb.  
+     * The Aqua scroll control works as follows.
+     * The scrollbar's value is the position of the left (or top) side of 
+     * the view area in the content area being scrolled.  
+     * The maximum value of the control is therefore the dimension of 
+     * the content area less the size of the view area.
+     * Since these values are all integers, and Tk gives the thumb position
+     * as fractions, we have introduced a scaling factor.
+     *   
      */
-    middle = scrollPtr->firstFraction / (scrollPtr->firstFraction +
-            (1.0 - scrollPtr->lastFraction));
+    dViewSize =  (scrollPtr->lastFraction - scrollPtr->firstFraction)
+            * SCROLLBAR_SCALING_DVALUE;
+    
+    viewSize = (SInt32) dViewSize;
+    controlMax = (SInt32) (SCROLLBAR_SCALING_DVALUE - dViewSize);
+    controlValue = (SInt32) (SCROLLBAR_SCALING_DVALUE * scrollPtr->firstFraction);
+
+    SetControlViewSize(macScrollPtr->sbHandle,viewSize);
+    SetControl32BitMaximum(macScrollPtr->sbHandle, controlMax);
+
+#if 0             
+    middle = scrollPtr->firstFraction /
+	    (1.0 - (scrollPtr->lastFraction - scrollPtr->firstFraction));
     
     viewSize = (SInt32) ((scrollPtr->lastFraction - scrollPtr->firstFraction) 
-            * MAX_SCROLLBAR_DVALUE);
-    
+            * SCROLLBAR_SCALING_DVALUE
+	    / (1.0 - (scrollPtr->lastFraction - scrollPtr->firstFraction)));
+
+  
     SetControlViewSize(macScrollPtr->sbHandle,viewSize);
-    SetControlValue(macScrollPtr->sbHandle, 
-            (short) (middle * MAX_SCROLLBAR_VALUE) );
+    SetControl32BitValue(macScrollPtr->sbHandle, 
+			 (middle) );
+#endif  
     contrlHilite = GetControlHilite(macScrollPtr->sbHandle);
-    SetControlMinimum(macScrollPtr->sbHandle, 0);
+    SetControl32BitMinimum(macScrollPtr->sbHandle, 0);
     if ( contrlHilite == 0 || contrlHilite == 255) {
         if (scrollPtr->firstFraction == 0.0 &&
                 scrollPtr->lastFraction == 1.0) {
-            SetControlMinimum(macScrollPtr->sbHandle, MAX_SCROLLBAR_VALUE);
+            SetControl32BitMinimum(macScrollPtr->sbHandle, SCROLLBAR_SCALING_VALUE);
         } else {
             HiliteControl(macScrollPtr->sbHandle, 0);
         }
     }
+    SetControl32BitValue(macScrollPtr->sbHandle, controlValue);
     
     if ( !IsControlVisible (macScrollPtr -> sbHandle) ) {
         SetControlVisibility(macScrollPtr->sbHandle,TRUE,FALSE);
