@@ -9,7 +9,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tkMacOSXSubwindows.c,v 1.2 2002/08/31 06:12:30 das Exp $
+ * RCS: @(#) $Id: tkMacOSXSubwindows.c,v 1.2.2.4 2005/03/12 00:49:28 wolfsuit Exp $
  */
 
 #include "tkInt.h"
@@ -26,7 +26,13 @@
  */
 static RgnHandle tmpRgn = NULL;
 
-static void UpdateOffsets _ANSI_ARGS_((TkWindow *winPtr, int deltaX, int deltaY));
+/*
+ * Prototypes for functions used only in this file.
+ */
+
+static void GenerateConfigureNotify (TkWindow *winPtr, int includeWin);
+static void UpdateOffsets _ANSI_ARGS_((TkWindow *winPtr, 
+        int deltaX, int deltaY));
 
 /*
  *----------------------------------------------------------------------
@@ -62,6 +68,31 @@ XDestroyWindow(
     
     
     if (Tk_IsTopLevel(macWin->winPtr)) {
+        WindowRef winRef;
+        /*
+         * We are relying on the Activate Mac OS event to pass the
+         * focus away from a window that is getting Destroyed to the
+         * Front non-floating window.  BUT we don't get activate events
+         * when a floating window is destroyed - since the front non-floating
+         * window doesn't in fact get activated...  So maybe we can check here
+         * and if we are destroying a floating window, we can pass the focus
+         * back to the front non-floating window...
+         */
+         
+        if (macWin->grafPtr != NULL) {
+            TkWindow *focusPtr = TkGetFocusWin(macWin->winPtr);
+            if (focusPtr == NULL || (focusPtr->mainPtr->winPtr == macWin->winPtr)) {
+                winRef = GetWindowFromPort(macWin->grafPtr);
+                if (TkpIsWindowFloating (winRef)) {
+                    Window window;
+                    
+                    window = TkMacOSXGetXWindow(FrontNonFloatingWindow());
+                    if (window != None) {
+                        TkMacOSXGenerateFocusEvent(window, 1);
+                    }
+                }
+            }
+        }
 	DisposeRgn(macWin->clipRgn);
 	DisposeRgn(macWin->aboveClipRgn);
 	
@@ -361,7 +392,45 @@ XResizeWindow(
 	UpdateOffsets(macWin->winPtr, deltaX, deltaY);
     }
 }
+
 
+/*
+ *----------------------------------------------------------------------
+ *
+ * GenerateConfigureNotify --
+ *
+ *	Generates ConfigureNotify events for all the child widgets
+ *      of the widget passed in the winPtr parameter.  If includeWin
+ *      is true, also generates ConfigureNotify event for the 
+ *      widget itself.
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	ConfigureNotify events will be posted.
+ *
+ *----------------------------------------------------------------------
+ */
+
+static void
+GenerateConfigureNotify (TkWindow *winPtr, int includeWin)
+{
+    TkWindow *childPtr;
+
+    for (childPtr = winPtr->childList; childPtr != NULL;
+                               childPtr = childPtr->nextPtr) {
+        if (!Tk_IsMapped(childPtr) || Tk_IsTopLevel(childPtr)) {
+            continue;
+        }
+        GenerateConfigureNotify(childPtr, 1);
+    }
+    if (includeWin) {
+        TkDoConfigureNotify(winPtr);
+    }
+}    
+
+
 /*
  *----------------------------------------------------------------------
  *
@@ -465,6 +534,7 @@ XMoveResizeWindow(
 	UpdateOffsets(macWin->winPtr, deltaX, deltaY);
 	TkMacOSXWinBounds(macWin->winPtr, &bounds);
 	InvalWindowRect(GetWindowFromPort(destPort),&bounds);
+        GenerateConfigureNotify(macWin->winPtr, 0);
     }
 }
 
@@ -566,6 +636,7 @@ XMoveWindow(
 	UpdateOffsets(macWin->winPtr, deltaX, deltaY);
 	TkMacOSXWinBounds(macWin->winPtr, &bounds);
 	InvalWindowRect(GetWindowFromPort(destPort),&bounds);
+	GenerateConfigureNotify(macWin->winPtr, 0);
     }
 }
 
@@ -898,8 +969,8 @@ TkMacOSXInvalidateWindow(
     WindowRef windowRef;
     CGrafPtr  grafPtr;
 
-    grafPtr=TkMacOSXGetDrawablePort((Drawable)macWin);
-    windowRef=GetWindowFromPort(grafPtr);
+    grafPtr = TkMacOSXGetDrawablePort((Drawable)macWin);
+    windowRef = GetWindowFromPort(grafPtr);
     
     if (flag == TK_WINDOW_ONLY) {
 	InvalWindowRgn(windowRef,macWin->clipRgn);
