@@ -9,7 +9,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tkUnixFont.c,v 1.18.2.2 2003/07/19 01:45:42 hobbs Exp $
+ * RCS: @(#) $Id: tkUnixFont.c,v 1.18.2.4 2005/04/12 22:05:47 hobbs Exp $
  */
  
 #include "tkUnixInt.h"
@@ -2054,7 +2054,7 @@ FontMapLoadPage(subFontPtr, row)
     int row;			/* Index of the page to be loaded into 
 				 * the cache. */
 {
-    char src[TCL_UTF_MAX], buf[16];
+    char buf[16], src[TCL_UTF_MAX];
     int minHi, maxHi, minLo, maxLo, scale, checkLo;
     int i, end, bitOffset, isTwoByteFont, n;
     Tcl_Encoding encoding;
@@ -2091,7 +2091,7 @@ FontMapLoadPage(subFontPtr, row)
     end = (row + 1) << FONTMAP_SHIFT;
     for (i = row << FONTMAP_SHIFT; i < end; i++) {
 	int hi, lo;
-	
+
 	if (Tcl_UtfToExternal(NULL, encoding, src, Tcl_UniCharToUtf(i, src),
         	TCL_ENCODING_STOPONERROR, NULL, buf, sizeof(buf), NULL, 
 		NULL, NULL) != TCL_OK) {
@@ -2716,6 +2716,13 @@ GetFontAttributes(display, fontStructPtr, faPtr)
     } else {
 	TkInitFontAttributes(&faPtr->fa);
 	TkInitXLFDAttributes(&faPtr->xa);
+    }
+    /*
+     * Do last ditch check for family.  It seems that some X servers can
+     * fail on the X font calls above, slipping through earlier checks.
+     * X-Win32 5.4 is one of these.
+     */
+    if (faPtr->fa.family == NULL) {
 	faPtr->fa.family = Tk_GetUid("");
 	faPtr->xa.foundry = Tk_GetUid("");
 	faPtr->xa.charset = Tk_GetUid("");
